@@ -7,7 +7,7 @@ use utf8;
 
 BEGIN {
 	$Web::Magic::Async::AUTHORITY = 'cpan:TOBYINK';
-	$Web::Magic::Async::VERSION   = '0.006';
+	$Web::Magic::Async::VERSION   = '0.007';
 }
 
 use AnyEvent::HTTP;
@@ -57,6 +57,12 @@ sub do_request
 	$self;
 }
 
+sub user_agent
+{
+	my ($self) = @_;
+	Web::Magic::Exception->throw('Web::Magic::Async does not use LWP::UserAgent');
+}
+
 sub _cancel_progress
 {
 	my ($self) = @_;
@@ -248,6 +254,11 @@ Usually, this will throw an error if called on a Web::Magic object that
 has already been requested. Asynchronous requests can be cancelled while
 they are still in progress, but not once they are complete.
 
+=item C<< user_agent >>
+
+Web::Magic::Async does not use LWP::UserAgent, so this method throws a
+Web::Magic::Exception.
+
 =back
 
 =head1 BUGS
